@@ -1,8 +1,10 @@
 """Python API for Sensibo."""
 from __future__ import annotations
 import asyncio
+from datetime import datetime, timezone
 
 import json
+import logging
 from typing import Any
 
 from aiohttp import ClientResponse, ClientSession
@@ -17,6 +19,8 @@ TIMEOUT = 5 * 60
 HTTP_AUTH_FAILED_STATUS_CODES = {401, 403}
 MAX_POSSIBLE_STEP = 1000
 
+LOGGER = logging.getLogger(__name__)
+
 
 class SensiboClient:
     """Sensibo client."""
@@ -61,7 +65,17 @@ class SensiboClient:
             name = dev["room"]["name"]
             measure: dict[str, Any] = dev["measurements"]
             temperature = measure.get("temperature")
+            feelslike = measure.get("feelsLike")
             humidity = measure.get("humidity")
+
+            # Add in new sensors for AirQ + Element model
+            tvoc = measure.get("tvoc")
+            co2 = measure.get("co2")
+
+            # Add information for Element model
+            etoh = measure.get("etoh")
+            iaq = measure.get("iaq")
+
             ac_states: dict[str, Any] = dev["acState"]
             target_temperature = ac_states.get("targetTemperature")
             hvac_mode = ac_states.get("mode")
@@ -157,14 +171,16 @@ class SensiboClient:
             pure_ac_integration: bool | None = None
             pure_geo_integration: bool | None = None
             pure_measure_integration: bool | None = None
+            pure_prime_integration: bool | None = None
             if dev["productModel"] == "pure":
                 pure_boost_enabled = pure_conf.get("enabled", False)
-                pure_sensitivity = pure_conf.get("sensitivity", "off")
+                pure_sensitivity = pure_conf.get("sensitivity", "n").lower()
                 pure_ac_integration = pure_conf.get("ac_integration", False)
                 pure_geo_integration = pure_conf.get("geo_integration", False)
                 pure_measure_integration = pure_conf.get(
                     "measurements_integration", False
                 )
+                pure_prime_integration = pure_conf.get("prime_integration", False)
             pm25 = measure.get("pm25")
 
             # Binary sensors for main device
@@ -179,8 +195,13 @@ class SensiboClient:
             )
             filter_clean: bool = filters.get("shouldCleanFilters", False)
             clean_time: dict[str, Any] = filters.get("lastFiltersCleanTime") or {}
-            filter_last_reset: str | None = (
-                clean_time.get("time") if clean_time else None
+            clean_time_str: str | None = clean_time.get("time")
+            filter_last_reset: datetime | None = (
+                datetime.strptime(clean_time_str, "%Y-%m-%dT%H:%M:%SZ").replace(
+                    tzinfo=timezone.utc
+                )
+                if clean_time_str
+                else None
             )
 
             # Timer
@@ -191,10 +212,17 @@ class SensiboClient:
             timer_time = None
             if dev["productModel"] != "pure":
                 timer_on = timer.get("isEnabled", False)
-            timer_id = timer.get("id")
-            timer_state: dict[str, Any] | None = timer.get("acState")
-            timer_state_on = timer_state.get("on") if timer_state else None
-            timer_time = timer.get("targetTime")
+                timer_id = timer.get("id")
+                timer_state: dict[str, Any] | None = timer.get("acState")
+                timer_state_on = timer_state.get("on") if timer_state else None
+                timer_time_str: str | None = timer.get("targetTime")
+                timer_time = (
+                    datetime.strptime(timer_time_str, "%Y-%m-%dT%H:%M:%S").replace(
+                        tzinfo=timezone.utc
+                    )
+                    if timer_time_str
+                    else None
+                )
 
             # Smartmode
             smart: dict[str, Any] = dev["smartMode"] if dev["smartMode"] else {}
@@ -212,14 +240,19 @@ class SensiboClient:
             schedules: dict[str, Schedules] = {}
             if schedule_list:
                 for schedule in schedule_list:
+                    next_utc_str: str = schedule["nextTime"]
+                    next_utc = datetime.strptime(
+                        next_utc_str, "%Y-%m-%dT%H:%M:%S"
+                    ).replace(tzinfo=timezone.utc)
                     schedules[schedule["id"]] = Schedules(
                         id=schedule["id"],
                         enabled=schedule["isEnabled"],
+                        name=schedule["name"],
                         state_on=schedule["acState"].get("on"),
                         state_full=schedule["acState"],
                         days=schedule["recurringDays"],
                         time=schedule["targetTimeLocal"],
-                        next_utc=schedule["nextTime"],
+                        next_utc=next_utc,
                     )
 
             device_data[unique_id] = SensiboDevice(
@@ -228,7 +261,10 @@ class SensiboClient:
                 name=name,
                 ac_states=ac_states,
                 temp=temperature,
+                feelslike=feelslike,
                 humidity=humidity,
+                tvoc=tvoc,
+                co2=co2,
                 target_temp=target_temperature,
                 hvac_mode=hvac_mode,
                 device_on=running,
@@ -261,6 +297,8 @@ class SensiboClient:
                 pure_ac_integration=pure_ac_integration,
                 pure_geo_integration=pure_geo_integration,
                 pure_measure_integration=pure_measure_integration,
+                pure_prime_integration=pure_prime_integration,
+                pure_conf=pure_conf,
                 pm25=pm25,
                 room_occupied=room_occupied,
                 update_available=update_available,
@@ -277,6 +315,8 @@ class SensiboClient:
                 smart_low_state=smart_low_state,
                 smart_high_state=smart_high_state,
                 schedules=schedules,
+                etoh=etoh,
+                iaq=iaq,
             )
 
         return SensiboData(raw=data, parsed=device_data)
@@ -320,7 +360,7 @@ class SensiboClient:
         return await self._put(APIV2 + "/pods/{}/smartmode".format(uid), params, data)
 
     async def async_set_climate_react(
-        self, uid: str, data: dict[str, bool]
+        self, uid: str, data: dict[str, Any]
     ) -> dict[str, Any]:
         """Set Climate React on a device.
 
@@ -328,7 +368,7 @@ class SensiboClient:
         data: dict according to dev["smartmode"]
         """
         params = {"apiKey": self.api_key}
-        return await self._put(APIV2 + "/pods/{}/smartmode".format(uid), params, data)
+        return await self._post(APIV2 + "/pods/{}/smartmode".format(uid), params, data)
 
     async def async_get_timer(self, uid: str) -> dict[str, Any]:
         """Get Timer on a device.
@@ -424,6 +464,17 @@ class SensiboClient:
             APIV2 + "/pods/{}/calibration/".format(uid), params, data
         )
 
+    async def async_set_pureboost(
+        self, uid: str, data: dict[str, float]
+    ) -> dict[str, Any]:
+        """Set Pure Boost Settings.
+
+        uid: UID for device
+        data: dict as pure_conf
+        """
+        params = {"apiKey": self.api_key}
+        return await self._put(APIV2 + "/pods/{}/pureboost".format(uid), params, data)
+
     async def async_set_ac_states(
         self,
         uid: str,
@@ -463,17 +514,21 @@ class SensiboClient:
         )
 
     async def _get(
-        self, path: str, params: dict[str, Any], retry: bool = False
+        self, path: str, params: dict[str, Any], retry: int = 3
     ) -> dict[str, Any]:
         """Make GET api call to Sensibo api."""
-        async with self._session.get(path, params=params, timeout=self.timeout) as resp:
-            try:
+        LOGGER.debug("Attempting get with path %s and parameters %s", path, params)
+        try:
+            async with self._session.get(
+                path, params=params, timeout=self.timeout
+            ) as resp:
                 return await self._response(resp)
-            except Exception as error:
-                if retry is False:
-                    await asyncio.sleep(5)
-                    return await self._get(path, params, True)
-                raise error
+        except Exception as error:
+            LOGGER.debug("Retry %d on path %s", 4 - retry, path)
+            if retry > 0:
+                await asyncio.sleep(7)
+                return await self._get(path, params, retry - 1)
+            raise error
 
     async def _put(
         self,
@@ -483,16 +538,16 @@ class SensiboClient:
         retry: bool = False,
     ) -> dict[str, Any]:
         """Make PUT api call to Sensibo api."""
-        async with self._session.put(
-            path, params=params, data=json.dumps(data), timeout=self.timeout
-        ) as resp:
-            try:
+        try:
+            async with self._session.put(
+                path, params=params, data=json.dumps(data), timeout=self.timeout
+            ) as resp:
                 return await self._response(resp)
-            except Exception as error:
-                if retry is False:
-                    await asyncio.sleep(5)
-                    return await self._put(path, params, data, True)
-                raise error
+        except Exception as error:
+            if retry is False:
+                await asyncio.sleep(5)
+                return await self._put(path, params, data, True)
+            raise error
 
     async def _post(
         self,
@@ -502,16 +557,17 @@ class SensiboClient:
         retry: bool = False,
     ) -> dict[str, Any]:
         """Make POST api call to Sensibo api."""
-        async with self._session.post(
-            path, params=params, data=json.dumps(data), timeout=self.timeout
-        ) as resp:
-            try:
+
+        try:
+            async with self._session.post(
+                path, params=params, data=json.dumps(data), timeout=self.timeout
+            ) as resp:
                 return await self._response(resp)
-            except Exception as error:
-                if retry is False:
-                    await asyncio.sleep(5)
-                    return await self._post(path, params, data, True)
-                raise error
+        except Exception as error:
+            if retry is False:
+                await asyncio.sleep(5)
+                return await self._post(path, params, data, True)
+            raise error
 
     async def _patch(
         self,
@@ -521,31 +577,33 @@ class SensiboClient:
         retry: bool = False,
     ) -> dict[str, Any]:
         """Make PATCH api call to Sensibo api."""
-        async with self._session.patch(
-            path, params=params, data=json.dumps(data), timeout=self.timeout
-        ) as resp:
-            try:
+
+        try:
+            async with self._session.patch(
+                path, params=params, data=json.dumps(data), timeout=self.timeout
+            ) as resp:
                 return await self._response(resp)
-            except Exception as error:
-                if retry is False:
-                    await asyncio.sleep(5)
-                    return await self._patch(path, params, data, True)
-                raise error
+        except Exception as error:
+            if retry is False:
+                await asyncio.sleep(5)
+                return await self._patch(path, params, data, True)
+            raise error
 
     async def _delete(
         self, path: str, params: dict[str, Any], retry: bool = False
     ) -> dict[str, Any]:
         """Make DELETE api call to Sensibo api."""
-        async with self._session.delete(
-            path, params=params, timeout=self.timeout
-        ) as resp:
-            try:
+
+        try:
+            async with self._session.delete(
+                path, params=params, timeout=self.timeout
+            ) as resp:
                 return await self._response(resp)
-            except Exception as error:
-                if retry is False:
-                    await asyncio.sleep(5)
-                    return await self._delete(path, params, True)
-                raise error
+        except Exception as error:
+            if retry is False:
+                await asyncio.sleep(5)
+                return await self._delete(path, params, True)
+            raise error
 
     async def _response(self, resp: ClientResponse) -> dict[str, Any]:
         """Return response from call."""
